﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;

namespace AntDesign
{
    public class AutoCompleteSearch : Search, IAutoCompleteInput
    {
        public AutoCompleteSearch()
        {
            AutoComplete = false;
            BindOnInput = true;
        }

        [CascadingParameter]
        public IAutoCompleteRef Component { get; set; }

        [CascadingParameter(Name = "OverlayTriggerContext")]
        public ForwardRef OverlayTriggerContext
        {
            get => RefBack;
            set
            {
                WrapperRefBack = value;
                RefBack = value;
            }
        }

        protected override void OnInitialized()
        {
            base.OnInitialized();

            if (Component != null) Component?.SetInputComponent(this);
        }

        internal override async Task OnFocusAsync(FocusEventArgs e)
        {
            if (Component != null) await Component?.InputFocus(e);

            await base.OnFocusAsync(e);
        }

        protected override async Task OnkeyDownAsync(KeyboardEventArgs args)
        {
            await base.OnkeyDownAsync(args);

            if (Component != null) await Component?.InputKeyDown(args);
        }

        protected override async Task OnInputAsync(ChangeEventArgs args)
        {
            await base.OnInputAsync(args);

            await Component?.InputInput(args);
        }

        protected override void OnValueChange(string value)
        {
            base.OnValueChange(value);

            Component?.InputValueChange(value);
        }

        #region IAutoCompleteInput

        public void SetValue(object value)
        {
            this.CurrentValue = value?.ToString();
        }

        #endregion IAutoCompleteInput
    }
}
